import processing.sound.*;

SoundFile bgMusic;
SoundFile steps;

PImage gameBg, introBg;
PImage hitboxes;
PImage hero;

PImage logo, msg, con;
PImage tmpStart, start, startPressed;
PImage tmpBack, back, backPressed;

PImage light, screen;
PGraphics lightMask;
PGraphics pg;
boolean lights;
int page = 0;

int x, y, initX, initY, tx, ty;
float dx = 0;
float dy = 0;
int dir=1;
float speed;
int frameNum;
int lightRadius;
float freq;
int heroWidth, heroHeight;
float scaleX, scaleY;
int btnWidth, btnHeight;

void setup() {
  fullScreen();

  //All the constant below are ratios used for keeping the game presentation intact in all different screen sizes
  //Scale X and Y are respective ratios used in the rest of the code
  lightRadius = width/19;
  heroWidth = width/32;
  heroHeight = heroWidth;
  scaleY = height/1080.0;
  scaleX = width/1920.0;
  btnWidth = int(width/4);
  btnHeight = int(btnWidth/3.318181818);

  //Load music
  bgMusic = new SoundFile(this, "music.mp3");
  bgMusic.loop();
  steps = new SoundFile(this, "steps.mp3");
  steps.amp(0.8);

  //Load images
  hitboxes = loadImage("hitboxes.jpg");
  hitboxes.resize(width, height);
  gameBg = loadImage("gameBg.jpg");
  gameBg.resize(width, height);  
  introBg = loadImage("introBg.jpg");
  introBg.resize(width, height);  
  hero = loadImage("hero.png");
  hero.resize(heroWidth*9, heroHeight*4);  
  logo = loadImage("logo.png");
  logo.resize(btnWidth, btnHeight); 
  msg = loadImage("msg.png");
  msg.resize(btnWidth, int(1.5*btnHeight)); 
  start = loadImage("start.png");
  start.resize(btnWidth, btnHeight);
  tmpStart = start; 
  startPressed = loadImage("startPressed.png");
  startPressed.resize(btnWidth, btnHeight); 
  con = loadImage("con.png");
  con.resize(btnWidth, btnHeight);
  back = loadImage("back.png");
  back.resize(btnWidth, btnHeight);
  tmpBack = back; 
  backPressed = loadImage("backPressed.png");
  backPressed.resize(btnWidth, btnHeight);

  initX=int(width*0.9453);
  initY=int(height*0.9305);
  x=initX;
  y=initY;

  speed=3.0*scaleX;
  freq=0.45*scaleX;
  
  //Set treasure location
  tx=int(width*0.5468);
  ty=int(height*0.6851);

  //Create hitboxes
  image(hitboxes, 0, 0);
  loadPixels();
}

void draw() {
  if (page == 0) {
    drawMenu();
  }
  else if (page == 1) {
    drawGame();
  }
  else if (page == 2) {
    drawCongrats();
  }
}

void drawMenu() {
  image(introBg, 0, 0);
  image(logo, width/2-logo.width/2, height/2-msg.height/2-logo.height/2-100*scaleY);
  image(msg, width/2-msg.width/2, height/2-msg.height/2);
  image(tmpStart, width/2-start.width/2, height/2+msg.height/2-start.height/2+100*scaleY);
  if (mouseX > width/2-start.width/2 && mouseX < width/2+start.width/2 && mouseY > height/2+msg.height/2+100*scaleY-start.height/2 && mouseY < height/2+msg.height/2+start.height/2+100*scaleY) {
    tmpStart = startPressed;
  } else {
    tmpStart = start;
  }
}

void drawGame() {
  image(gameBg, 0, 0);
  screen = get(0, 0, width, height);
  light();
  image(screen, 0, 0);
  drawHero();
  didTouchHitbox();
  makeSteps();
  endGame();
}

void drawCongrats() {
  image(gameBg, 0, 0);
  drawHero();
  
  image(con, width/2-con.width/2, height/2-back.height/2-con.height-50*scaleY);
  image(tmpBack, width/2-back.width/2, height/2-back.height/2);
  if (mouseX > width/2-back.width/2 && mouseX < width/2+back.width/2 && mouseY > height/2-back.height/2 && mouseY < height/2+back.height/2) {
    tmpBack = backPressed;
  } else {
    tmpBack = back;
  }
}

void mousePressed() {
  if (page == 0 && mouseX > width/2-start.width/2 && mouseX < width/2+start.width/2 && mouseY > height/2+msg.height/2+100*scaleY-start.height/2 && mouseY < height/2+msg.height/2+start.height/2+100*scaleY) {
    tmpBack = back;
    noCursor();
    page=1;
  }
  if (page == 2 && mouseX > width/2-back.width/2 && mouseX < width/2+back.width/2 && mouseY > height/2-back.height/2 && mouseY < height/2+back.height/2) {
    tmpStart = start;
    page=0;
    x=initX;
    y=initY;
  }
}

void drawHero() {
  if (dx!=0 || dy!=0) {
    frameNum = int((frameCount*freq))%9;
  }
  switch(dir) {
  case 0: 
    //down
    image(hero, x-heroWidth/2, y-heroHeight/2, heroWidth, heroHeight, 0+frameNum*heroWidth, heroHeight*2, heroWidth+frameNum*heroWidth, heroHeight*3);
    break;
  case 1: 
    //up
    image(hero, x-heroWidth/2, y-heroHeight/2, heroWidth, heroHeight, 0+frameNum*heroWidth, 0, heroWidth+frameNum*heroWidth, heroHeight);
    break;
  case 2: 
    //left
    image(hero, x-heroWidth/2, y-heroHeight/2, heroWidth, heroHeight, 0+frameNum*heroWidth, heroHeight, heroWidth+frameNum*heroWidth, heroHeight*2);
    break;
  case 3: 
    //right
    image(hero, x-heroWidth/2, y-heroHeight/2, heroWidth, heroHeight, 0+frameNum*heroWidth, heroHeight*3, heroWidth+frameNum*heroWidth, heroHeight*4);
    break;
  }
}

//  Drawing and masking out light
void light() {
  lightMask = createGraphics(width, height);
  fill(0, 250);
  noStroke();
  rect(0, 0, width, height);
  pg = lightMask;
  pg.beginDraw();
  pg.background(0, 0);
  pg.blendMode(ADD);
  pg.noStroke();
  drawLight();
  pg.endDraw();
  screen.mask(lightMask);
}

void endGame() {
  if (x>tx-25 && x<tx+25 && y>ty-25 && y<ty+25) {
    page = 2;
    cursor();
    steps.stop();
  }
}

//Compare the x,y to the corresponding position in the hitbox graphic
void didTouchHitbox() {
  if ((pixels[x+(y+1)*width-int(scaleX*20)]!=color(255, 255, 255) || pixels[x+int(y+20*scaleY)*width-int(scaleX*20)]!=color(255, 255, 255)) && dir==2) {}
  else if ((pixels[x+(y+1)*width+int(scaleX*20)]!=color(255, 255, 255) || pixels[x+int(y+20*scaleY)*width+int(scaleX*20)]!=color(255, 255, 255)) && dir==3) {}
  else if ((pixels[x+y*width-int(5*scaleX)]!=color(255, 255, 255) || pixels[x+y*width+int(5*scaleX)]!=color(255, 255, 255)) && dir==1) {}
  else if ((pixels[x+(y+int(30*scaleY))*width-int(5*scaleX)]!=color(255, 255, 255) || pixels[x+(y+int(30*scaleY))*width+int(5*scaleX)]!=color(255, 255, 255)) && dir==0) {}
  else {
    x+=dx;
    y+=dy;
  }
}

void drawLight() {
  for (int i=0; i<=lightRadius/2; i++) {
    pg.fill(10);
    switch(dir) {
    case 0: 
      pg.ellipse(x, y+lightRadius*3/4, lightRadius-i*2, 1.5*lightRadius-i*2);
      break;
    case 1: 
      pg.ellipse(x, y-lightRadius*2/5, lightRadius-i*2, 1.5*lightRadius-i*2);
      break;
    case 2: 
      pg.ellipse(x-lightRadius*3/5, y, 1.5*lightRadius-i*2, lightRadius-i*2);
      break;
    case 3: 
      pg.ellipse(x+lightRadius*3/5, y, 1.5*lightRadius-i*2, lightRadius-i*2);
      break;
    }
  }
}

void makeSteps() {
  if ((dx!=0 || dy!=0) && !steps.isPlaying()) {
    steps.play();
  } else if (steps.isPlaying()) {
    steps.pause();
  }
}

void keyReleased() {
  if (keyCode==DOWN) {
    dy=0;
  } else if (keyCode==UP) {
    dy=0;
  } else if (keyCode==LEFT) {
    dx=0;
  } else if (keyCode==RIGHT) {
    dx=0;
  }
}

void keyPressed() {
  if (page == 1) {
    if (keyCode==DOWN) {
      dir=0;
      dx=0;
      dy=speed;
    } else if (keyCode==UP) {
      dir=1;
      dx=0;
      dy=-speed;
    } else if (keyCode==LEFT) {
      dir=2;
      dx=-speed;
      dy=0;
    } else if (keyCode==RIGHT) {
      dir=3;
      dx=speed;
      dy=0;
    }
  }
}
